typedef unsigned int u32;
typedef int s32;
typedef unsigned short u16;
typedef short s16;
typedef unsigned char u8;
typedef short s8;

#ifdef RMCP

#define SYSTEM__RESOURCE_MANAGER__SP_INSTANCE 0x809bd738
#define SYSTEM__RESOURCE_MANAGER__GET_FILE 0x805411fc
#define ENGINE_CLASS_PTR 0x809bd728

#endif

#ifdef RMCE

#define SYSTEM__RESOURCE_MANAGER__SP_INSTANCE 0x809b8f78
#define SYSTEM__RESOURCE_MANAGER__GET_FILE 0x8053bcc0
#define ENGINE_CLASS_PTR 0x809b8f68

#endif

#ifdef RMCJ

#define SYSTEM__RESOURCE_MANAGER__SP_INSTANCE 0x809bc798
#define SYSTEM__RESOURCE_MANAGER__GET_FILE 0x80540b7c
#define ENGINE_CLASS_PTR 0x809bc788

#endif

#ifdef RMCK

#define SYSTEM__RESOURCE_MANAGER__SP_INSTANCE 0x809abd78
#define SYSTEM__RESOURCE_MANAGER__GET_FILE 0x8052f254
#define ENGINE_CLASS_PTR 0x809abd68

#endif

typedef struct
{
    //char	magic[4];	// always LEX_BIN_MAGIC
	u32 magic;
    u16		major_version;	// usually LEX_MAJOR_VERSION
    u16		minor_version;	// usually LEX_MINOR_VERSION
    u32		size;		// size of this file (header+streams)
    u32		element_off;	// offset of first lex_element_t, 32-bit aligned
}lex_header_t;

typedef struct
{
    u32		magic;		// identification of section
    u32		size;		// size of 'data' (this header excluded)
    u8		data[];		// section data, 32-bit aligned
}lex_element_t;
//https://wiki.tockdom.com/wiki/LEX_(File_Format)/C_example

void *get_data0(void);

#define DEFAULT_TIME_LIMIT 150

#define LEX_BIN_MAGIC 0x4C452D58
#define CTDN_MAGIC 0x4354444E
#define TERMINATOR_MAGIC 0

unsigned char getEngineClass(void* engineClassPtr);

unsigned int getCountDownTimeLimitSecondsFromLex(void *lexFile){
	if(!lexFile)return DEFAULT_TIME_LIMIT;
	lex_header_t *header = lexFile;
	if(header->magic != LEX_BIN_MAGIC || header->major_version != 1 || header->minor_version != 0)return DEFAULT_TIME_LIMIT; //invalid lex header
	void *currentElementVoidPtr = ((unsigned char*)lexFile) + header->element_off;
	lex_element_t *currentElement;
	while(1){
		currentElement = (lex_element_t*)currentElementVoidPtr;
		if(currentElement->magic == CTDN_MAGIC){//CTDN section
			if(currentElement->size < 0xC)return DEFAULT_TIME_LIMIT;//invalid section size
			//現在は200cc, 300cc, バトルのことを考慮しない
			//Currently, we do not consider 200cc, 300cc, battle
			unsigned char engineClass = getEngineClass((void*)ENGINE_CLASS_PTR);
			if(engineClass > 2)return DEFAULT_TIME_LIMIT;
			unsigned short* times = (void*)(currentElement->data);
			return times[engineClass + 1];
		}
		if(currentElement->magic == TERMINATOR_MAGIC) return DEFAULT_TIME_LIMIT;//terminator
		currentElementVoidPtr = ((unsigned char*)currentElementVoidPtr) + currentElement->size + 8;
	}
    return DEFAULT_TIME_LIMIT;
}

void getCountDownTimeLimitSeconds(void){
	unsigned int *countDownTimeLimitSeconds = (void*)0x800011D4;
	void *resourceManager = *((void**)((void*)SYSTEM__RESOURCE_MANAGER__SP_INSTANCE));
	void *(*System__ResourceManager__getFile)(void*, unsigned int, char*, unsigned int*) = (void*)SYSTEM__RESOURCE_MANAGER__GET_FILE;
	//void *lexFile = System__ResourceManager__getFile(1, "course.lex", 0);
	//https://github.com/riidefi/mkw/blob/master/source/game/system/ResourceManager.hpp
	//https://github.com/riidefi/mkw/blob/master/source/game/system/ResourceManager.cpp
	void *lexFile = System__ResourceManager__getFile(resourceManager, 1, get_data0(), 0);
	*countDownTimeLimitSeconds = getCountDownTimeLimitSecondsFromLex(lexFile);
}